# pylint: disable=non-parent-init-called, too-many-public-methods
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.widgets.view.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTKBaseView Module."""

# Standard Library Imports
import locale
from typing import Any, Dict, List, Tuple

# Third Party Imports
# noinspection PyPackageRequirements
import treelib
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gdk, Gtk, _

# RAMSTK Local Imports
from .button import do_make_buttonbox
from .dialog import RAMSTKMessageDialog
from .label import RAMSTKLabel
from .panel import RAMSTKPanel
from .treeview import RAMSTKTreeView


# noinspection PyUnresolvedReferences
class RAMSTKBaseView(Gtk.HBox):
    """Metaclass for all RAMSTK ListView, ModuleView, and WorkView classes.

    Attributes of the RAMSTKBaseView are:

    :cvar RAMSTK_USER_CONFIGURATION: the instance of the RAMSTK Configuration
        class.
    :cvar dic_tab_position: dictionary holding the Gtk.PositionType()s for
        each of left, right, top, and bottom.

    The following attributes are used to build toolbars and pop-up menus:

    :ivar _dic_icons: dictionary containing icon name and absolute path
        key:value pairs.

    :ivar _lst_callbacks: the list of callback functions to associate with
        toolbutton items and pop-up menu entries on a view.
    :ivar _lst_icons: the list of icon names (keys) in the _dic_icons
        dictionary to use for each toolbutton item and pop-up menu entry.
    :ivar _lst_mnu_labels: the list of labels to use on pop-up menu entries
        for a view.  These are generally only used for views with a
        RAMSTKTreeView().
    :ivar _lst_tooltips: the list of tooltips to associate with each
        toolbutton item.

    :ivar float _mission_time: the mission time for the open RAMSTK Program.
    :ivar _notebook: the Gtk.Notebook() to hold all the pages of information to
        be displayed.
    :type _notebook: :class:`Gtk.Notebook`
    :ivar int _revision_id: the ID of the Revision associated with the
        information being displayed in the View.
    :ivar int _parent_id: the ID of the parent object associated with the
        information being displayed in the View.
    :ivar treeview: the Gtk.TreeView() to display the information for the View.
    :type treeview: :class:`Gtk.TreeView`
    :ivar str fmt: the formatting code for numerical displays.
    :ivar hbx_tab_label: the Gtk.HBox() containing the View's Gtk.Notebook()
        tab Gtk.Label().
    :type hbx_tab_label: :class:`Gtk.HBox`
    """

    # Define private class scalar attributes.
    _pixbuf: bool = False

    # Define public class dict attributes.
    dic_tab_position = {
        "left": Gtk.PositionType.LEFT,
        "right": Gtk.PositionType.RIGHT,
        "top": Gtk.PositionType.TOP,
        "bottom": Gtk.PositionType.BOTTOM,
    }

    # Define public class scalar attributes.
    RAMSTK_USER_CONFIGURATION: RAMSTKUserConfiguration = None  # type: ignore

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the RAMSTK Base View.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__()

        self.RAMSTK_USER_CONFIGURATION = configuration
        self.RAMSTK_LOGGER = logger
        self.RAMSTK_LOGGER.do_create_logger(
            __name__, self.RAMSTK_USER_CONFIGURATION.RAMSTK_LOGLEVEL, to_tty=False
        )

        # Initialize private dictionary attributes.
        self._dic_icons: Dict[str, str] = self.__set_icons()

        # Initialize private list attributes.
        self._lst_callbacks: List[object] = [
            self.do_request_update,
            self.do_request_update_all,
        ]
        self._lst_icons: List[str] = [
            "save",
            "save-all",
        ]
        self._lst_mnu_labels: List[str] = [
            _("Save"),
            _("Save All"),
        ]
        self._lst_tooltips: List[str] = []
        self._lst_handler_id: List[int] = []
        self._lst_layouts: List[str] = [
            "allocation",
            "failure_definition",
            "fmea",
            "function",
            "hardware",
            "hazard",
            "incident",
            "pof",
            "requirement",
            "revision",
            "similar_item",
            "software",
            "stakeholder",
            "testing",
            "validation",
        ]

        # Initialize private scalar attributes.
        self._img_tab: Gtk.Image = Gtk.Image()
        self._mission_time: float = float(self.RAMSTK_USER_CONFIGURATION.RAMSTK_MTIME)
        self._notebook: Gtk.Notebook = Gtk.Notebook()
        self._pnlPanel: RAMSTKPanel = RAMSTKPanel()
        self._revision_id: int = 0
        self._tree_loaded: bool = False

        # Initialize public dictionary attributes.
        self.dic_pkeys: Dict[str, int] = {
            "revision_id": 0,
            f"{self._tag}_id": 0,
            "parent_id": 0,
            "record_id": 0,
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.fmt: str = (
            "{0:0." + str(self.RAMSTK_USER_CONFIGURATION.RAMSTK_DEC_PLACES) + "G}"
        )
        self.hbx_tab_label: Gtk.HBox = Gtk.HBox()

        try:
            locale.setlocale(
                locale.LC_ALL, self.RAMSTK_USER_CONFIGURATION.RAMSTK_LOCALE
            )
        except locale.Error as _error:
            locale.setlocale(locale.LC_ALL, "")
            self.RAMSTK_LOGGER.do_log_exception(__name__, _error)

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_set_cursor_active, "request_set_cursor_active")
        pub.subscribe(self.do_set_cursor_active, f"succeed_update_{self._tag}")
        pub.subscribe(self.do_set_cursor_active, f"succeed_calculate_{self._tag}")
        pub.subscribe(self.do_set_cursor_active, "succeed_update_all")
        pub.subscribe(self.do_set_cursor_active_on_fail, f"fail_calculate_{self._tag}")
        pub.subscribe(self.do_set_cursor_active_on_fail, f"fail_delete_{self._tag}")
        pub.subscribe(self.do_set_cursor_active_on_fail, f"fail_insert_{self._tag}")
        pub.subscribe(self.do_set_cursor_active_on_fail, f"fail_update_{self._tag}")

        pub.subscribe(self.on_select_revision, "selected_revision")

    def do_embed_treeview_panel(self) -> None:
        """Embed a treeview RAMSTKPanel() into the layout.

        :return: None
        """
        _fmt_file = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_CONF_DIR
            + "/layouts/"
            + self.RAMSTK_USER_CONFIGURATION.RAMSTK_FORMAT_FILE[self._tag]
        )

        try:
            _bg_color = self.RAMSTK_USER_CONFIGURATION.RAMSTK_COLORS[self._tag + "bg"]
            _fg_color = self.RAMSTK_USER_CONFIGURATION.RAMSTK_COLORS[self._tag + "fg"]
        except KeyError:
            _bg_color = "#FFFFFF"
            _fg_color = "#000000"

        self._pnlPanel.do_make_treeview(
            bg_color=_bg_color,
            fg_color=_fg_color,
            fmt_file=_fmt_file,
            attrs=self._pnlPanel.dic_attribute_widget_map,
        )

        self.pack_end(self._pnlPanel, True, True, 0)

        self.show_all()

    def do_make_layout(self) -> None:
        """Create a view with the following layout.

        +-----+---------------------------------------+
        |  B  |                WIDGETS                |
        |  U  |                                       |
        |  T  |                                       |
        |  T  |                                       |
        |  O  |                                       |
        |  N  |                                       |
        |  S  |                                       |
        +-----+---------------------------------------+
        self.make_toolbuttons -------> self

        :return: None
        :rtype: None
        """
        self.make_tab_label(tablabel=self._tablabel, tooltip=self._tabtooltip)
        self.make_toolbuttons(
            icons=self._lst_icons,  # type: ignore
            tooltips=self._lst_tooltips,  # type: ignore
            callbacks=self._lst_callbacks,  # type: ignore
        )

    def do_make_layout_lr(self) -> Gtk.HPaned:
        """Create a view with the following layout.

        +-----+-------------------+-------------------+
        |  B  |      L. SIDE      |      R. SIDE      |
        |  U  |                   |                   |
        |  T  |                   |                   |
        |  T  |                   |                   |
        |  O  |                   |                   |
        |  N  |                   |                   |
        |  S  |                   |                   |
        +-----+-------------------+-------------------+
        self.make_toolbuttons -----+--> self
                                   |
                      _hpaned -----+

        :return: _hpaned; the Gtk.HPaned() that creates the left and right
            sides for further population.
        :rtype: :class:`Gtk.Hpaned`
        """
        self.do_make_layout()

        _hpaned: Gtk.HPaned = Gtk.HPaned()

        self.pack_start(_hpaned, True, True, 0)

        return _hpaned

    def do_make_layout_lrr(self) -> Tuple[Gtk.HPaned, Gtk.VPaned]:
        """Create a view with the following layout.

        +-----+-------------------+-------------------+
        |  B  |      L. SIDE      |      R. TOP       |
        |  U  |                   |                   |
        |  T  |                   |                   |
        |  T  |                   +-------------------+
        |  O  |                   |     R. BOTTOM     |
        |  N  |                   |                   |
        |  S  |                   |                   |
        +-----+-------------------+-------------------+
           self.make_toolbuttons  -----+--> self
                                       |
        _vpaned_right -----> _hpaned --+

        :return: (_hpaned, _vpaned_right); the Gtk.HPaned() and Gtk.Vpaned()
            that create the left and right sections for further population.
        :rtype: tuple
        """
        self.do_make_layout()

        _hpaned: Gtk.HPaned = Gtk.HPaned()
        _vpaned_right: Gtk.VPaned = Gtk.VPaned()

        _hpaned.pack2(_vpaned_right, True, True)

        self.pack_start(_hpaned, True, True, 0)

        return _hpaned, _vpaned_right

    def do_make_layout_llr(self) -> Tuple[Gtk.HPaned, Gtk.VPaned]:
        """Create a view with the following layout.

        +-----+-------------------+-------------------+
        |  B  |       L. TOP      |     R. SIDE       |
        |  U  |                   |                   |
        |  T  |                   |                   |
        |  T  +-------------------+                   |
        |  O  |     L. BOTTOM     |                   |
        |  N  |                   |                   |
        |  S  |                   |                   |
        +-----+-------------------+-------------------+
           self.make_toolbuttons  -----+--> self
                                       |
         _vpaned_left -----> _hpaned --+

        :return: (_hpaned, _vpaned_left); the Gtk.HPaned() and Gtk.Vpaned()
            that create the left and right sections for further population.
        :rtype: tuple
        """
        self.do_make_layout()

        _hpaned: Gtk.HPaned = Gtk.HPaned()
        _vpaned_left: Gtk.VPaned = Gtk.VPaned()

        _hpaned.pack1(_vpaned_left, True, True)

        self.pack_start(_hpaned, True, True, 0)

        return _hpaned, _vpaned_left

    def do_make_layout_llrr(self) -> Tuple[Gtk.VPaned, Gtk.VPaned]:
        """Create a view with the following layout.

        +-----+-------------------+-------------------+
        |  B  |       L. TOP      |      R. TOP       |
        |  U  |                   |                   |
        |  T  |                   |                   |
        |  T  +-------------------+-------------------+
        |  O  |     L. BOTTOM     |     R. BOTTOM     |
        |  N  |                   |                   |
        |  S  |                   |                   |
        +-----+-------------------+-------------------+
           self.make_toolbuttons  -----+--> self
                                       |
         _vpaned_left --+--> _hpaned --+
                        |
        _vpaned_right --+

        :return: (_vpaned_left, _vpaned_right); the two Gtk.Vpaned()
            that create the left and right sections for further population.
        :rtype: tuple
        """
        self.do_make_layout()

        _hpaned: Gtk.HPaned = Gtk.HPaned()
        _vpaned_left: Gtk.VPaned = Gtk.VPaned()
        _vpaned_right: Gtk.VPaned = Gtk.VPaned()

        _hpaned.pack1(_vpaned_left, True, True)
        _hpaned.pack2(_vpaned_right, True, True)

        self.pack_start(_hpaned, True, True, 0)

        return _vpaned_left, _vpaned_right

    @staticmethod
    def do_raise_dialog(**kwargs: Any) -> RAMSTKMessageDialog:
        """Raise a dialog in response to information, warnings, and errors.

        This method will display a message dialog of the appropriate severity
        information, warning, or error containing a message to the user.

        :return: _dialog
        """
        _parent = kwargs.get("parent", None)

        _dialog = RAMSTKMessageDialog(parent=_parent)

        return _dialog

    def do_request_delete(self, __button: Gtk.ToolButton) -> None:
        """Request to delete selected record from the RAMSTKFunction table.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        _parent = self.get_parent().get_parent().get_parent().get_parent().get_parent()
        _prompt = _(
            f"You are about to delete {self._tag.title()} "
            f"{self.dic_pkeys['record_id']} and all data associated with it.  Is this "
            f"really what you want to do?"
        )
        _dialog = RAMSTKMessageDialog(parent=_parent)
        _dialog.do_set_message(_prompt)
        _dialog.do_set_message_type("question")

        if _dialog.do_run() == Gtk.ResponseType.YES:
            self.do_set_cursor_busy()
            pub.sendMessage(
                f"request_delete_{self._tag}",
                node_id=self.dic_pkeys["record_id"],
            )

        _dialog.do_destroy()

    def do_request_insert(self, **kwargs: Any) -> None:
        """Request insert a new work stream element into the program database.

        :return: None
        """
        _sibling = kwargs.get("sibling", True)

        self.do_set_cursor_busy()

        if _sibling:
            pub.sendMessage(
                f"request_insert_{self._tag.lower()}",
                attributes=self.dic_pkeys,
            )
        else:
            _parent_id = self.dic_pkeys["parent_id"]
            self.dic_pkeys["parent_id"] = self.dic_pkeys["record_id"]
            pub.sendMessage(
                f"request_insert_{self._tag.lower()}",
                attributes=self.dic_pkeys,
            )  # noqa
            self.dic_pkeys["parent_id"] = _parent_id

    def do_request_insert_child(self, __button: Gtk.ToolButton) -> Any:
        """Request to insert a new child entity of the selected entity.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        return self.do_request_insert(sibling=False)

    def do_request_insert_sibling(self, __button: Gtk.ToolButton) -> Any:
        """Send request to insert a new sibling entity.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        return self.do_request_insert(sibling=True)

    def do_request_update(self, __button: Gtk.ToolButton) -> None:
        """Request to update selected record to RAMSTK program database.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        self.do_set_cursor_busy()
        pub.sendMessage(
            f"request_update_{self._tag}", node_id=self.dic_pkeys["record_id"]
        )

    def do_request_update_all(self, __button: Gtk.ToolButton) -> None:
        """Send request to save all the records to RAMSTK program database.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        self.do_set_cursor_busy()
        pub.sendMessage(f"request_update_all_{self._tag}s")

    def do_set_cursor(self, cursor: Gdk.CursorType) -> None:
        """Set the cursor for the Module, List, and Work Book Gdk.Window().

        :param cursor: the Gdk.Cursor.new() to set.  Only handles one of the
                       following:
                       - Gdk.CursorType.X_CURSOR
                       - Gdk.CursorType.ARROW
                       - Gdk.CursorType.CENTER_PTR
                       - Gdk.CIRCLE
                       - Gdk.CROSS
                       - Gdk.CROSS_REVERSE
                       - Gdk.CursorType.CROSSHAIR
                       - Gdk.DIAMOND_CROSS
                       - Gdk.DOUBLE_ARROW
                       - Gdk.DRAFT_LARGE
                       - Gdk.DRAFT_SMALL
                       - Gdk.EXCHANGE
                       - Gdk.FLEUR
                       - Gdk.GUMBY
                       - Gdk.HAND1
                       - Gdk.HAND2
                       - Gdk.CursorType.LEFT_PTR - non-busy cursor
                       - Gdk.PENCIL
                       - Gdk.PLUS
                       - Gdk.QUESTION_ARROW
                       - Gdk.CursorType.RIGHT_PTR
                       - Gdk.SB_DOWN_ARROW
                       - Gdk.SB_H_DOUBLE_ARROW
                       - Gdk.SB_LEFT_ARROW
                       - Gdk.SB_RIGHT_ARROW
                       - Gdk.SB_UP_ARROW
                       - Gdk.SB_V_DOUBLE_ARROW
                       - Gdk.TCROSS
                       - Gdk.TOP_LEFT_ARROW
                       - Gdk.CursorType.WATCH - when application is busy
                       - Gdk.XTERM - selection bar
        :type cursor: :class:`Gdk.Cursor`
        :return: None
        :rtype: None
        """
        try:
            # noinspection PyCallByClass,PyArgumentList
            self.get_parent_window().set_cursor(Gdk.Cursor.new(cursor))
            Gdk.flush()
        except AttributeError:
            # There is no parent window.
            pass

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def do_set_cursor_active(self, tree: treelib.Tree = "") -> None:
        """Set active cursor for the Module, List, and Work Book Gdk.Window().

        :param tree: the treelib Tree() passed in the PyPubSub message.  Only
            needed when this method is a PyPubSub subscriber.
        :return: None
        """
        self.do_set_cursor(Gdk.CursorType.LEFT_PTR)

    def do_set_cursor_active_on_fail(self, error_message: str = "") -> None:
        """Set active cursor for the Module, List, and Work Book Gdk.Window().

        This method will also write the error message to the RAMSTK debug_log to (
        hopefully) assist in troubleshooting.

        :param error_message: the error message broadcast with the
        'fail' message.  Only needed when this method is a PyPubSub subscriber.
        :return: None
        :rtype: None
        """
        self.do_set_cursor(Gdk.CursorType.LEFT_PTR)

        self.RAMSTK_LOGGER.do_log_debug(__name__, error_message)

    def do_set_cursor_busy(self) -> None:
        """Set busy cursor for the Module, List, and Work Book Gdk.Window().

        :return: None
        :rtype: None
        """
        self.do_set_cursor(Gdk.CursorType.WATCH)

    # noinspection PyUnusedLocal
    # pylint: disable=unused-argument
    def make_tab_label(self, **kwargs: Dict[str, Any]) -> None:
        """Make the view's tab label.

        :return: None
        """
        try:
            self._img_tab.set_from_file(self._dic_icons["tab"])
            self.hbx_tab_label.pack_start(self._img_tab, True, True, 0)
        except KeyError:
            # There is no icon to display on the tab.  Just move along.
            pass

        _label: RAMSTKLabel = RAMSTKLabel(self._tablabel)
        _label.do_set_properties(
            height=30,
            width=-1,
            justify=Gtk.Justification.CENTER,
            tooltip=self._tabtooltip,
        )
        self.hbx_tab_label.pack_end(_label, True, True, 0)
        self.hbx_tab_label.show_all()

    def make_toolbuttons(self, **kwargs: Dict[str, Any]) -> None:
        """Create the RAMSTKBaseView() tool buttons.

        Keyword arguments are passed along to the do_make_buttonbox() function.

        :return: None
        :rtype: None
        """
        _scrolledwindow = Gtk.ScrolledWindow()
        _scrolledwindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        _scrolledwindow.add_with_viewport(do_make_buttonbox(self, **kwargs))
        self.pack_start(_scrolledwindow, False, False, 0)

    def on_button_press(self, treeview: RAMSTKTreeView, event: Gdk.EventButton) -> None:
        """Handle mouse clicks on the View's RTKTreeView().

        :param treeview: the RAMSTKTreeView() that called this method.  If is
            unused in this method.
        :param event: the Gdk.Event() that called this method (the important
        attribute is which mouse button was clicked).

                      * 1 = left
                      * 2 = scrollwheel
                      * 3 = right
                      * 4 = forward
                      * 5 = backwards

        :return: None
        """
        treeview.handler_block(treeview.dic_handler_id["button-press"])

        if event.button == 3:
            _menu = Gtk.Menu()
            _menu.popup_at_pointer(event)

            # pylint: disable=unused-variable
            for _idx, __ in enumerate(self._lst_icons):
                _menu_item = Gtk.ImageMenuItem()
                _image = Gtk.Image()
                _image.set_from_file(self._dic_icons[self._lst_icons[_idx]])
                _menu_item.set_label(self._lst_mnu_labels[_idx])
                _menu_item.set_image(_image)
                _menu_item.set_property("use_underline", True)
                _menu_item.connect(
                    "activate",
                    self._lst_callbacks[_idx],
                )
                _menu_item.show()
                _menu.append(_menu_item)

        treeview.handler_unblock(treeview.dic_handler_id["button-press"])

    def on_insert(self, node_id: int = 0, tree: treelib.Tree = "") -> None:
        """Add row to the RAMSTKTreeView for newly added element.

        :param node_id: the ID of the newly added element.
        :param tree: the treelib Tree() containing the work stream module's
            data.
        :return: None
        :rtype: None
        """
        _data = tree.get_node(node_id).data[self._tag].get_attributes()
        self._pnlPanel.on_insert(_data)

    def on_select_revision(self, attributes: Dict[str, Any]) -> None:
        """Set the Revision ID when a new Revision is selected.

        :return: None
        :rtype: None
        """
        self._revision_id = attributes["revision_id"]
        self.dic_pkeys["revision_id"] = attributes["revision_id"]

    def __set_icons(self) -> Dict[str, str]:
        """Set the dict of icons.

        :return: the dict of icons to use in RAMSTK.
        :rtype: dict
        """
        return {
            "action": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/action.png",
            "add": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/add.png",
            "assembly": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/assembly.png",
            "calculate": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/calculate.png",
            "calculate_all": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/calculate-all.png",
            "cancel": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/cancel.png",
            "cause": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/cause.png",
            "complete": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/complete.png",
            "control": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/control.png",
            "chart": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/charts.png",
            "edit": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/edit.png",
            "environment": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/environment.png",
            "error": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/error.png",
            "export": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/export.png",
            "important": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/important.png",
            "insert_child": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/insert_child.png",
            "insert_sibling": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/insert_sibling.png",
            "mechanism": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/mechanism.png",
            "mission": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/mission.png",
            "mode": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/mode.png",
            "none": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/none.png",
            "opload": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/load.png",
            "opstress": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/stress.png",
            "part": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/part.png",
            "partial": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/partial.png",
            "mission_phase": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/phase.png",
            "plot": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/charts.png",
            "question": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/question.png",
            "refresh-view": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/view-refresh.png",
            "remove": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/remove.png",
            "rollup": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/rollup.png",
            "reports": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/reports.png",
            "save": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/save.png",
            "save-all": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/save-all.png",
            "save-layout": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/save-layout.png",
            "testmethod": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/method.png",
            "warning": self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR
            + "/32x32/warning.png",
        }


class RAMSTKListView(RAMSTKBaseView):
    """Class to display list and matrix type data in the RAMSTK List Book.

    This is the meta class for all RAMSTK List View classes.  Attributes of the
    RAMSTKListView are:

    :ivar tab_label: the Gtk.Label() displaying text for the List View tab.
    :type tab_label: :class:`Gtk.Label`
    """

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the List View.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, super().do_request_insert_sibling)
        self._lst_icons.insert(0, "add")

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.tab_label: Gtk.Label = Gtk.Label()

        # Subscribe to PyPubSub messages.

    def make_ui(self) -> None:
        """Build the list view user interface.

        :return: None
        """
        super().do_make_layout()
        super().do_embed_treeview_panel()


class RAMSTKModuleView(RAMSTKBaseView):
    """Display data in the RAMSTK Module Book.

    This is the meta class for all RAMSTK Module View classes. Attributes of the
    RAMSTKModuleView are:
    """

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the RAMSTKModuleView meta-class.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["insert_part"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/insert_part.png"
        )
        self._dic_key_index: Dict[str, int] = {}

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, super().do_request_insert_sibling)
        self._lst_callbacks.insert(1, super().do_request_delete)
        self._lst_icons.insert(0, "add")
        self._lst_icons.insert(1, "remove")

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

    def make_ui(self) -> None:
        """Build the user interface for a ModuleView.

        :return: None
        :rtype: None
        """
        super().do_make_layout()
        super().do_embed_treeview_panel()


class RAMSTKWorkView(RAMSTKBaseView):
    """Class to display data in the RAMSTK Work Book.

    This is the meta class for all RAMSTK Work View classes.  Attributes of the
    RAMSTKWorkView are:
    """



# pylint: disable=unused-import, missing-docstring
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.fmea.__init__.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK GTK3 FMEA package."""

# RAMSTK Local Imports
from .panel import FMEAMethodPanel, FMEATreePanel
from .view import FMEAWorkView



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.components.inductor.py is part of the RAMSTK
#       Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Inductor Results Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.milhdbk217f import MilHdbk217FResultPanel
from ramstk.views.gtk3.widgets import RAMSTKEntry


class InductorMilHdbk217FResultPanel(MilHdbk217FResultPanel):
    """Displays Inductor assessment results attribute data.

    The Inductor assessment result view displays all the assessment results
    for the selected inductor.  This includes, currently, results for
    MIL-HDBK-217FN2 parts count and part stress methods.  The attributes of an
    Inductor assessment result view are:

    :cvar dict _dic_part_stress: dictionary of MIL-HDBK-217F part stress
        models.  The key is the subcategory ID attribute of the component.

    :ivar list _lst_labels: list of label text to display for the capacitor
        MIL-HDBK-217 input parameters.

    :ivar self.txtLambdaB: displays the base hazard rate for the Hardware
        item.
    :ivar txtPiC: displays the construction factor for the Hardware item.
    """

    # Define private dict class attributes.
    _dic_part_stress: Dict[int, str] = {
        1: '<span foreground="blue">\u03BB<sub>p</sub> = '
        "\u03BB<sub>b</sub>\u03C0<sub>C</sub>\u03C0<sub>Q</sub>\u03C0"
        "<sub>E</sub></span>",
        2: '<span foreground="blue">\u03BB<sub>p</sub> = '
        "\u03BB<sub>b</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
    }

    # Define private list attributes.

    # Define private scalar class attributes.
    _title: str = _("Inductive Device MIL-HDBK-217F Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Inductor assessment result view."""
        super().__init__()

        # Initialize widgets.
        self.txtPiC: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dict attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_model": [
                13,
                self.lblModel,
                "",
                None,
                "",
                "",
                {
                    "tooltip": _(
                        "The assessment model used to calculate the inductive device's "
                        "failure rate."
                    ),
                },
                "",
            ],
            "lambda_b": [
                23,
                self.txtLambdaB,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _("The base hazard rate for the inductive device."),
                },
                "\u03BB<sub>b</sub>:",
            ],
            "pi_q": [
                30,
                self.txtPiQ,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The quality factor for the inductive device."),
                },
                "\u03C0<sub>Q</sub>:",
            ],
            "pi_e": [
                19,
                self.txtPiE,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The environment factor for the inductive device."),
                },
                "\u03C0<sub>E</sub>:",
            ],
            "pi_c": [
                13,
                self.txtPiC,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The construction factor for the inductive device."),
                },
                "\u03C0<sub>C</sub>:",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()

        # Subscribe to PyPubSub messages.

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected capacitor.

        :return: None
        :rtype: None
        """
        super().do_load_entries(attributes)

        self.txtPiC.set_sensitive(False)

        # MIL-HDBK-217F, Parts Stress
        if self.category_id == 5 and self._hazard_rate_method_id == 2:
            self.lblModel.do_update(self._dic_part_stress[self.subcategory_id])

            self.txtPiC.do_update(
                str(self.fmt.format(attributes["piC"])),
                signal="changed",
            )



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.validation.panel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Validation Panels."""

# Standard Library Imports
from datetime import date
from typing import Dict, List, Tuple

# Third Party Imports
import treelib
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gdk, Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKButton,
    RAMSTKComboBox,
    RAMSTKDateSelect,
    RAMSTKEntry,
    RAMSTKFixedPanel,
    RAMSTKSpinButton,
    RAMSTKTextView,
    RAMSTKTreePanel,
)


class ValidationTreePanel(RAMSTKTreePanel):
    """Panel to display flat list of validation tasks."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_validations"
    _tag = "validation"
    _title = _("Verification Task List")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Validation panel."""
        super().__init__()

        # Initialize private dictionary class attributes.
        self.tvwTreeView.dic_row_loader = {
            "validation": self.__do_load_verification,
        }

        # Initialize private list class attributes.
        self._lst_measurement_units: List[str] = []
        self._lst_verification_types: List[str] = []

        # Initialize private scalar class attributes.

        # Initialize public dictionary class attributes.
        self.dic_attribute_widget_map = {
            "revision_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Revision ID"),
                "gint",
            ],
            "validation_id": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Validation ID"),
                "gint",
            ],
            "acceptable_maximum": [
                2,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Acceptable Max."),
                "gfloat",
            ],
            "acceptable_mean": [
                3,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Acceptable Mean"),
                "gfloat",
            ],
            "acceptable_minimum": [
                4,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Acceptable Min."),
                "gfloat",
            ],
            "acceptable_variance": [
                5,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Acceptable Variance"),
                "gfloat",
            ],
            "confidence": [
                6,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                95.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Confidence"),
                "gfloat",
            ],
            "cost_average": [
                7,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Expected Cost"),
                "gfloat",
            ],
            "cost_ll": [
                8,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Cost LCL"),
                "gfloat",
            ],
            "cost_maximum": [
                9,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Maximum Cost"),
                "gfloat",
            ],
            "cost_mean": [
                10,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mean Cost"),
                "gfloat",
            ],
            "cost_minimum": [
                11,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Minimum Cost"),
                "gfloat",
            ],
            "cost_ul": [
                12,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Cost UCL"),
                "gfloat",
            ],
            "cost_variance": [
                13,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Cost Vaiance"),
                "gfloat",
            ],
            "date_end": [
                14,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                date.today(),
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("End Date"),
                "gchararray",
            ],
            "date_start": [
                15,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                date.today(),
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Start Date"),
                "gchararray",
            ],
            "description": [
                16,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Description"),
                "gchararray",
            ],
            "measurement_unit": [
                17,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Unit of Measure"),
                "gchararray",
            ],
            "name": [
                18,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Code"),
                "gchararray",
            ],
            "status": [
                19,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("% Complete"),
                "gfloat",
            ],
            "task_specification": [
                20,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Specification"),
                "gchararray",
            ],
            "task_type": [
                21,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Type"),
                "gchararray",
            ],
            "time_average": [
                22,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Expected Task Time"),
                "gfloat",
            ],
            "time_ll": [
                23,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Time LCL"),
                "gfloat",
            ],
            "time_maximum": [
                24,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Maximum Task Time"),
                "gfloat",
            ],
            "time_mean": [
                25,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mean Task Time"),
                "gfloat",
            ],
            "time_minimum": [
                26,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Minimum Task Time"),
                "gfloat",
            ],
            "time_ul": [
                27,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Time UCL"),
                "gfloat",
            ],
            "time_variance": [
                28,
                Gtk.CellRendererText(),
                "edited",
                None,
                "mvw_editing_validation",
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Task Time Variance"),
                "gfloat",
            ],
        }

        # Initialize public list class attributes.

        # Initialize public scalar class attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(
            _("Displays the hierarchical list of validations.")
        )

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._on_module_switch, "mvwSwitchedPage")

    def do_load_measurement_units(
        self, measurement_unit: Dict[int, Tuple[str, str]]
    ) -> None:
        """Load the verification task measurement unit list.

        :param measurement_unit: the dict containing the units of measure.
        :return: None
        """
        self._lst_measurement_units = [""]

        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["measurement_unit"]
        ).get_cells()[0]
        _cell.set_property("has-entry", False)
        _cellmodel = _cell.get_property("model")
        _cellmodel.clear()
        _cellmodel.append([""])

        # pylint: disable=unused-variable
        for __, _key in enumerate(measurement_unit):
            self._lst_measurement_units.append(measurement_unit[_key][1])
            _cellmodel.append([measurement_unit[_key][1]])

    def do_load_verification_types(
        self, verification_type: Dict[int, Tuple[str, str]]
    ) -> None:
        """Load the verification task type list.

        :param verification_type: the dict containing the verification task
            types.
        :return: None
        """
        self._lst_verification_types = [""]

        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["task_type"]
        ).get_cells()[0]
        _cell.set_property("has-entry", False)
        _cellmodel = _cell.get_property("model")
        _cellmodel.clear()
        _cellmodel.append([""])

        # pylint: disable=unused-variable
        for __, _key in enumerate(verification_type):
            self._lst_verification_types.append(verification_type[_key][1])
            _cellmodel.append([verification_type[_key][1]])

    def _on_module_switch(self, module: str = "") -> None:
        """Respond to changes in selected Module View module (tab).

        :param module: the name of the module that was just selected.
        :return: None
        """
        _model, _row = self.tvwTreeView.selection.get_selected()

        if module == self._tag and _row is not None:
            _code = _model.get_value(_row, self.tvwTreeView.position["validation_id"])
            _name = _model.get_value(_row, self.tvwTreeView.position["name"])
            _title = _(f"Analyzing Validation {_code}: {_name}")

            pub.sendMessage("request_set_title", title=_title)

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the Validation Module View RAMSTKTreeView().

        This method is called whenever a Validation Module View
        RAMSTKTreeView() row is activated/changed.

        :param selection: the Validation class Gtk.TreeSelection().
        :return: None
        """
        _attributes = super().on_row_change(selection)

        if _attributes:
            self._record_id = _attributes["validation_id"]

            _title = _("Analyzing Verification Task {0}").format(
                str(_attributes["name"])
            )

            pub.sendMessage(
                "selected_validation",
                attributes=_attributes,
            )
            pub.sendMessage(
                "request_get_validation_attributes",
                node_id=self._record_id,
                table="validation",
            )
            pub.sendMessage(
                "request_set_title",
                title=_title,
            )

    def __do_load_verification(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter:
        """Load a verification task into the RAMSTKTreeView().

        :param node: the treelib Node() with the mode data to load.
        :param row: the parent row of the mode to load into the requirement
            tree.
        :return: _new_row; the row that was just populated with requirement
            data.
        :rtype: :class:`Gtk.TreeIter`
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        _measurement_unit = self._lst_measurement_units[_entity.measurement_unit]
        _task_type = self._lst_verification_types[_entity.task_type]

        _attributes = [
            _entity.revision_id,
            _entity.validation_id,
            _entity.acceptable_maximum,
            _entity.acceptable_mean,
            _entity.acceptable_minimum,
            _entity.acceptable_variance,
            _entity.confidence,
            _entity.cost_average,
            _entity.cost_ll,
            _entity.cost_maximum,
            _entity.cost_mean,
            _entity.cost_minimum,
            _entity.cost_ul,
            _entity.cost_variance,
            str(_entity.date_end),
            str(_entity.date_start),
            _entity.description,
            _measurement_unit,
            _entity.name,
            _entity.status,
            _entity.task_specification,
            _task_type,
            _entity.time_average,
            _entity.time_ll,
            _entity.time_maximum,
            _entity.time_mean,
            _entity.time_minimum,
            _entity.time_ul,
            _entity.time_variance,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _error_msg = _(
                "An error occurred when loading verification task {0} in the "
                "verification task list.  This might indicate it was missing "
                "it's data package, some of the data in the package was "
                "missing, or some of the data was the wrong type.  Row data "
                "was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg",
                logger_name="WARNING",
                message=_error_msg,
            )

        return _new_row


class ValidationTaskDescriptionPanel(RAMSTKFixedPanel):
    """Panel to display general data about the selected Validation task."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field = "validation_id"
    _select_msg = "selected_validation"
    _tag = "validation"
    _title = _("Verification Task Description")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Validation Task Description panel."""
        super().__init__()

        # Initialize widgets.
        self.btnEndDate: RAMSTKButton = RAMSTKButton()
        self.btnStartDate: RAMSTKButton = RAMSTKButton()
        self.cmbTaskType: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbMeasurementUnit: RAMSTKComboBox = RAMSTKComboBox()
        self.spnStatus: RAMSTKSpinButton = RAMSTKSpinButton()
        self.txtTaskID: RAMSTKEntry = RAMSTKEntry()
        self.txtCode: RAMSTKEntry = RAMSTKEntry()
        self.txtMaxAcceptable: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanAcceptable: RAMSTKEntry = RAMSTKEntry()
        self.txtMinAcceptable: RAMSTKEntry = RAMSTKEntry()
        self.txtVarAcceptable: RAMSTKEntry = RAMSTKEntry()
        self.txtSpecification: RAMSTKEntry = RAMSTKEntry()
        self.txtTask: RAMSTKTextView = RAMSTKTextView(Gtk.TextBuffer())
        self.txtEndDate: RAMSTKEntry = RAMSTKEntry()
        self.txtStartDate: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.
        self._dic_task_types: Dict[int, List[str]] = {}
        self._dic_units: Dict[int, str] = {}

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map = {
            "validation_id": [
                1,
                self.txtTaskID,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 50,
                    "editable": False,
                },
                _("Task ID:"),
                "gint",
            ],
            "name": [
                18,
                self.txtCode,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                "",
                {
                    "width": 50,
                    "editable": False,
                    "tooltip": _("Displays the ID of the selected V&amp;V activity."),
                },
                _("Task Code:"),
                "gchararray",
            ],
            "description": [
                16,
                self.txtTask,
                "changed",
                super().on_changed_textview,
                "mvw_editing_validation",
                "",
                {
                    "height": 100,
                    "width": 500,
                    "tooltip": _(
                        "Displays the description of the selected V&amp;V activity."
                    ),
                },
                _("Task Description:"),
                "gchararray",
            ],
            "task_type": [
                21,
                self.cmbTaskType,
                "changed",
                super().on_changed_combo,
                "mvw_editing_validation",
                "",
                {
                    "tooltip": _(
                        "Selects and displays the type of task for the selected "
                        "V&amp;V activity."
                    ),
                },
                _("Task Type:"),
                "gint",
            ],
            "task_specification": [
                20,
                self.txtSpecification,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                "",
                {
                    "tooltip": _(
                        "Displays the internal or industry specification or procedure "
                        "governing the selected V&amp;V activity."
                    ),
                },
                _("Specification:"),
                "gchararray",
            ],
            "measurement_unit": [
                17,
                self.cmbMeasurementUnit,
                "changed",
                super().on_changed_combo,
                "mvw_editing_validation",
                "",
                {
                    "tooltip": _(
                        "Selects and displays the measurement unit for the selected "
                        "V&amp;V activity acceptance parameter."
                    ),
                },
                _("Measurement Unit:"),
                "gint",
            ],
            "acceptable_minimum": [
                4,
                self.txtMinAcceptable,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the minimum acceptable value for the selected "
                        "V&amp;V activity."
                    ),
                },
                _("Min. Acceptable:"),
                "gfloat",
            ],
            "acceptable_maximum": [
                2,
                self.txtMaxAcceptable,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the maximum acceptable value for the selected "
                        "V&amp;V activity."
                    ),
                },
                _("Max. Acceptable:"),
                "gfloat",
            ],
            "acceptable_mean": [
                3,
                self.txtMeanAcceptable,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the mean acceptable value for the selected V&amp;V "
                        "activity."
                    ),
                },
                _("Mean Acceptable:"),
                "gfloat",
            ],
            "acceptable_variance": [
                5,
                self.txtVarAcceptable,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the acceptable variance for the selected V&amp;V "
                        "activity."
                    ),
                },
                _("Variance:"),
                "gfloat",
            ],
            "date_start": [
                15,
                self.txtStartDate,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                date.today(),
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the date the selected V&amp;V activity is scheduled "
                        "to start."
                    ),
                },
                _("Start Date:"),
                "gchararray",
            ],
            "date_end": [
                14,
                self.txtEndDate,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                date.today(),
                {
                    "width": 100,
                    "tooltip": _(
                        "Displays the date the selected V&amp;V activity is scheduled "
                        "to end."
                    ),
                },
                _("Start End:"),
                "gchararray",
            ],
            "status": [
                19,
                self.spnStatus,
                "value-changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "limits": [0, 0, 100, 1, 0.1],
                    "numeric": True,
                    "ticks": True,
                    "tooltip": _(
                        "Displays % complete of the selected V&amp;V activity."
                    ),
                },
                _("% Complete:"),
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.btnEndDate.connect(
            "button-release-event", self._do_select_date, self.txtEndDate
        )
        self.btnStartDate.connect(
            "button-release-event", self._do_select_date, self.txtStartDate
        )
        self.cmbTaskType.connect("changed", self._do_make_task_code)

        # Subscribe to PyPubSub messages.

    def do_load_measurement_units(
        self, measurement_unit: Dict[int, Tuple[str, str]]
    ) -> None:
        """Load the measurement units RAMSTKComboBox().

        :param measurement_unit: the list of measurement units to load.  The
            key is an integer representing the ID field in the database.  The
            value is a tuple with a unit abbreviation, unit name, and generic
            unit type.  For example:

            ('lbf', 'Pounds Force', 'unit')

        :return: None
        :rtype: None
        """
        _model = self.cmbMeasurementUnit.get_model()
        _model.clear()

        _units = []
        for _index, _key in enumerate(measurement_unit):
            self._dic_units[_index + 1] = measurement_unit[_key][1]
            _units.append([measurement_unit[_key][1]])
        self.cmbMeasurementUnit.do_load_combo(entries=_units)

    def do_load_validation_types(
        self, validation_type: Dict[int, Tuple[str, str]]
    ) -> None:
        """Load the validation task types RAMSTKComboBox().

        :param validation_type: a dict of validation task types.  The key is an
            integer representing the ID field in the database.  The value is a
            tuple with a task code, task name, and generic task type.  For
            example:

            ('RAA', 'Reliability, Assessment', 'validation')

        :return: None
        :rtype: None
        """
        _model = self.cmbTaskType.get_model()
        _model.clear()

        _task_types = []
        for _index, _key in enumerate(validation_type):
            self._dic_task_types[_index + 1] = [
                validation_type[_key][0],
                validation_type[_key][1],
            ]
            _task_types.append([validation_type[_key][1]])
        self.cmbTaskType.do_load_combo(entries=_task_types)

    def _do_make_task_code(self, combo: RAMSTKComboBox) -> None:
        """Create the validation task code.

        This method builds the task code based on the task type and the task
        ID.  The code created has the form:

            task type 3-letter abbreviation-task ID

        :param combo: the RAMSTKComboBox() that called this method.
        :return: None
        :rtype: None
        """
        try:
            _index = combo.get_active()

            _task_type = self._dic_task_types[_index][0]
            _task_code = f"{_task_type}-{self._record_id:04d}"

            self.txtCode.do_update(str(_task_code), signal="changed")

            pub.sendMessage(
                "wvw_editing_validation",
                node_id=[self._record_id, -1, ""],
                package={"name": _task_code},
            )
        except (AttributeError, KeyError):
            pass

    @staticmethod
    def _do_select_date(
        __button: RAMSTKButton, __event: Gdk.Event, entry: RAMSTKEntry
    ) -> str:
        """Request to launch a date selection dialog.

        This method is used to select the validation date for the Validation.

        :param __button: the ramstk.RAMSTKButton() that called this method.
        :type __button: :class:`ramstk.gui.gtk.ramstk.RAMSTKButton`
        :param __event: the Gdk.Event() that called this method.
        :type __event: :class:`Gdk.Event`
        :param entry: the Gtk.Entry() that the new date should be displayed in.
        :type entry: :class:`Gtk.Entry`
        :return: _date; the date in ISO-8601 (YYYY-mm-dd) format.
        :rtype: str
        """
        _dialog: RAMSTKDateSelect = RAMSTKDateSelect()

        _date = _dialog.do_run()
        _dialog.do_destroy()

        entry.set_text(str(_date))

        return _date


class ValidationTaskEffortPanel(RAMSTKFixedPanel):
    """Panel to display effort data about the selected Validation task."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field = "validation_id"
    _select_msg = "selected_validation"
    _tag = "validation"
    _title = _("Verification Task Effort")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Validation Task Effort panel."""
        super().__init__()

        # Initialize widgets.
        self.txtMinTime: RAMSTKEntry = RAMSTKEntry()
        self.txtExpTime: RAMSTKEntry = RAMSTKEntry()
        self.txtMaxTime: RAMSTKEntry = RAMSTKEntry()
        self.txtMinCost: RAMSTKEntry = RAMSTKEntry()
        self.txtExpCost: RAMSTKEntry = RAMSTKEntry()
        self.txtMaxCost: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanTimeLL: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanTime: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanTimeUL: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanCostLL: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanCost: RAMSTKEntry = RAMSTKEntry()
        self.txtMeanCostUL: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.
        self._dic_task_types: Dict[int, List[str]] = {}

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map = {
            "time_minimum": [
                26,
                self.txtMinTime,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Minimum person-time needed to complete the selected task."
                    ),
                },
                _("Min. Task Time:"),
                "gfloat",
            ],
            "time_average": [
                22,
                self.txtExpTime,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Most likely person-time needed to complete the selected task."
                    ),
                },
                _("Most Likely Task Time:"),
                "gfloat",
            ],
            "time_maximum": [
                24,
                self.txtMaxTime,
                "changed",
                super().on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _(
                        "Maximum person-time needed to complete the selected task."
                    ),
                },
                _("Max. Task Time:"),
                "gfloat",
            ],
            "time_ll": [
                23,
                self.txtMeanTimeLL,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                "",
                "gfloat",
            ],
            "time_mean": [
                25,
                self.txtMeanTime,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                _("Task Time (95% Confidence):"),
                "gfloat",
            ],
            "time_ul": [
                27,
                self.txtMeanTimeUL,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                "",
                "gfloat",
            ],
            "cost_minimum": [
                11,
                self.txtMinCost,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _("Minimim cost of the selected task."),
                },
                _("Min. Task Cost:"),
                "gfloat",
            ],
            "cost_average": [
                7,
                self.txtExpCost,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _("Most likely cost of the selected task."),
                },
                _("Most Likely Task Cost:"),
                "gfloat",
            ],
            "cost_maximum": [
                9,
                self.txtMaxCost,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "tooltip": _("Maximum cost of the selected task."),
                },
                _("Max. Task Cost:"),
                "gfloat",
            ],
            "cost_ll": [
                8,
                self.txtMeanCostLL,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                "",
                "gfloat",
            ],
            "cost_mean": [
                10,
                self.txtMeanCost,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                _("Task Cost (95% Confidence):"),
                "gfloat",
            ],
            "cost_ul": [
                12,
                self.txtMeanCostUL,
                "changed",
                self.on_changed_entry,
                "mvw_editing_validation",
                0.0,
                {
                    "width": 100,
                    "editable": False,
                },
                "",
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        self.__do_adjust_widgets()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.

    def do_load_validation_types(
        self, validation_type: Dict[int, Tuple[str, str]]
    ) -> None:
        """Load the validation task types RAMSTKComboBox().

        :param validation_type: a dict of validation task types.  The key is an
            integer representing the ID field in the database.  The value is a
            tuple with a task code, task name, and generic task type.  For
            example:

            ('RAA', 'Reliability, Assessment', 'validation')

        :return: None
        :rtype: None
        """
        for _index, _key in enumerate(validation_type):
            self._dic_task_types[_index + 1] = [
                validation_type[_key][0],
                validation_type[_key][1],
            ]

    def _do_load_code(self, task_code: int) -> None:
        """Load the Validation code RAMSTKEntry().

        :param task_code: the Validation code to load.
        :return: None
        :rtype: None
        """
        self.txtCode.do_update(str(task_code), signal="changed")

    def _do_make_task_code(self, task_type: str) -> str:
        """Create the validation task code.

        This method builds the task code based on the task type and the task
        ID.  The code created has the form:

            task type 3-letter abbreviation-task ID

        :param task_type: the three letter abbreviation for the task type.
        :return: _code
        :rtype: str
        """
        _code = ""

        # pylint: disable=unused-variable
        for __, _type in self._dic_task_types.items():
            if _type[1] == task_type:
                _code = f"{_type[0]}-{self._record_id:04d}"

        pub.sendMessage(
            "wvw_editing_validation",
            node_id=[self._record_id, -1, ""],
            package={"name": _code},
        )

        return _code

    @staticmethod
    def _do_select_date(
        __button: RAMSTKButton, __event: Gdk.Event, entry: RAMSTKEntry
    ) -> str:
        """Request to launch a date selection dialog.

        This method is used to select the validation date for the Validation.

        :param __button: the ramstk.RAMSTKButton() that called this method.
        :type __button: :class:`ramstk.gui.gtk.ramstk.RAMSTKButton`
        :param __event: the Gdk.Event() that called this method.
        :type __event: :class:`Gdk.Event`
        :param entry: the Gtk.Entry() that the new date should be displayed in.
        :type entry: :class:`Gtk.Entry`
        :return: _date; the date in ISO-8601 (YYYY-mm-dd) format.
        :rtype: str
        """
        _dialog: RAMSTKDateSelect = RAMSTKDateSelect()

        _date = _dialog.do_run()
        _dialog.do_destroy()

        entry.set_text(str(_date))

        return _date

    def _on_calculate_task(self, tree: treelib.Tree) -> None:
        """Wrap _do_load_panel() on successful task calculation.

        :param tree: the validation treelib.Tree().
        :return: None
        :rtype: None
        """
        _attributes = tree.get_node(self._record_id).data["validation"].get_attributes()
        self._do_load_panel(_attributes)

    def __do_adjust_widgets(self) -> None:
        """Adjust position of some widgets.

        :return: None
        :rtype: None
        """
        _fixed: Gtk.Fixed = self.get_children()[0].get_children()[0].get_children()[0]

        _time_entry: RAMSTKEntry = _fixed.get_children()[9]
        _cost_entry: RAMSTKEntry = _fixed.get_children()[21]

        # We add the mean time and mean time UL to the same y position as
        # the mean time LL widget.
        _x_pos: int = _fixed.child_get_property(_time_entry, "x")
        _y_pos: int = _fixed.child_get_property(_time_entry, "y")
        _fixed.move(self.txtMeanTimeLL, _x_pos, _y_pos)
        _fixed.move(self.txtMeanTime, _x_pos + 175, _y_pos)
        _fixed.move(self.txtMeanTimeUL, _x_pos + 350, _y_pos)

        # We add the mean cost and mean cost UL to the same y position as
        # the mean cost LL widget.
        _x_pos = _fixed.child_get_property(_cost_entry, "x")
        _y_pos = _fixed.child_get_property(_cost_entry, "y")
        _fixed.move(self.txtMeanCostLL, _x_pos, _y_pos)
        _fixed.move(self.txtMeanCost, _x_pos + 175, _y_pos)
        _fixed.move(self.txtMeanCostUL, _x_pos + 350, _y_pos)



# Standard Library Imports
import gettext
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKSiteConfiguration as RAMSTKSiteConfiguration
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.db import BaseDatabase as BaseDatabase
from ramstk.db import do_create_program_db as do_create_program_db
from ramstk.models import RAMSTKSiteInfoRecord as RAMSTKSiteInfoRecord
from ramstk.models.commondb import RAMSTKRPN as RAMSTKRPN
from ramstk.models.commondb import RAMSTKCategory as RAMSTKCategory
from ramstk.models.commondb import RAMSTKFailureMode as RAMSTKFailureMode
from ramstk.models.commondb import RAMSTKGroup as RAMSTKGroup
from ramstk.models.commondb import RAMSTKHazards as RAMSTKHazards
from ramstk.models.commondb import RAMSTKLoadHistory as RAMSTKLoadHistory
from ramstk.models.commondb import RAMSTKManufacturer as RAMSTKManufacturer
from ramstk.models.commondb import RAMSTKMeasurement as RAMSTKMeasurement
from ramstk.models.commondb import RAMSTKMethod as RAMSTKMethod
from ramstk.models.commondb import RAMSTKModel as RAMSTKModel
from ramstk.models.commondb import RAMSTKStakeholders as RAMSTKStakeholders
from ramstk.models.commondb import RAMSTKStatus as RAMSTKStatus
from ramstk.models.commondb import RAMSTKSubCategory as RAMSTKSubCategory
from ramstk.models.commondb import RAMSTKType as RAMSTKType
from ramstk.models.commondb import RAMSTKUser as RAMSTKUser

_ = gettext.gettext

class RAMSTKCommonDB:
    site_configuration: RAMSTKSiteConfiguration
    common_dao: BaseDatabase
    def __init__(self) -> None: ...
    def do_create_common(
        self, common_db: BaseDatabase, database: Dict[str, str], license_file: str
    ) -> None: ...
    def do_add_administrator(self) -> None: ...
    def do_load_site_info(self, license_file: str) -> None: ...
    def do_load_site_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_action_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_hardware_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_incident_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_miscellaneous_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_pof_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_requirement_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_rpn_variables(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_severity(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...
    def _do_load_user_workgroups(
        self, user_configuration: RAMSTKUserConfiguration
    ) -> RAMSTKUserConfiguration: ...



# Standard Library Imports
from typing import Any, Callable, Dict, List

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class FailureDefinitionTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: str
    _dic_row_loader: Dict[str, Callable]
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _on_module_switch(self, module: str = ...) -> None: ...
    _record_id: int
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def __do_load_failure_definition(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...



# * coding: utf8 *
#
#       ramstk.models.commondb.RAMSTKCondition.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007  2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKCondition Table Module."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKCondition(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_condition in RAMSTK Common database."""

    __defaults__ = {"description": "Condition Description", "condition_type": ""}
    __tablename__ = "ramstk_condition"
    __table_args__ = {"extend_existing": True}

    condition_id = Column(
        "fld_condition_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    condition_type = Column(
        "fld_condition_type", String(256), default=__defaults__["condition_type"]
    )

    def get_attributes(self):
        """Retrieve current values of RAMSTKCondition data model attributes.

        :return: {condition_id, description, condition_type} pairs
        :rtype: dict
        """
        _attributes = {
            "condition_id": self.condition_id,
            "description": self.description,
            "condition_type": self.condition_type,
        }

        return _attributes



# pylint: disable=unused-import, wrong-import-position
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.milhdbk217f.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 MIL-HDBK-217F Package."""

from .panel import MilHdbk217FResultPanel  # isort: skip

# RAMSTK Local Imports
from .components.capacitor import CapacitorMilHdbk217FResultPanel
from .components.connection import ConnectionMilHdbk217FResultPanel
from .components.inductor import InductorMilHdbk217FResultPanel
from .components.integrated_circuit import ICMilHdbk217FResultPanel
from .components.meter import MeterMilHdbk217FResultPanel
from .components.miscellaneous import MiscellaneousMilHdbk217FResultPanel
from .components.relay import RelayMilHdbk217FResultPanel
from .components.resistor import ResistorMilHdbk217FResultPanel
from .components.semiconductor import SemiconductorMilHdbk217FResultPanel
from .components.switch import SwitchMilHdbk217FResultPanel



# -*- coding: utf-8 -*-
#
#       ramstk.exim.imports.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK Import module."""

# Standard Library Imports
import inspect
import math
from collections import OrderedDict
from datetime import date
from typing import Any, Dict

# Third Party Imports
# noinspection PyPackageRequirements
import numpy as np

# noinspection PyPackageRequirements
import pandas as pd

# noinspection PyPackageRequirements
from dateutil import parser
from pubsub import pub

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase
from ramstk.exceptions import DataAccessError
from ramstk.models import (
    RAMSTKAllocationRecord,
    RAMSTKDesignElectricRecord,
    RAMSTKDesignMechanicRecord,
    RAMSTKFunctionRecord,
    RAMSTKHardwareRecord,
    RAMSTKMilHdbk217FRecord,
    RAMSTKNSWCRecord,
    RAMSTKReliabilityRecord,
    RAMSTKRequirementRecord,
    RAMSTKSimilarItemRecord,
    RAMSTKValidationRecord,
)


def _do_replace_nan(value: Any, default: Any) -> Any:
    """Check for NaN values and replace any with the default value.

    :param value: the value to check for NaN.
    :param default: the default value to replace NaN.
    :return: _value; the passed value or the default if the passed value
        was NaN.
    """
    _value = value

    # Check for Python NaN's.
    try:
        if math.isnan(value):
            _value = default
    # Non-numeric values raise a type error, so we can just move along.
    except TypeError:
        pass

    # Check for numpy NaN's.
    if value is np.nan:
        _value = default

    return _value


def _get_input_value(
    mapper: Dict[str, Any], df_row: pd.Series, field: str, default: Any
) -> Any:
    """Retrieve the input value for a field from the Pandas dataframe.

    :param mapper: the field mapping dict to use as the Rosetta stone.
    :param df_row: the row from the pandas DataFrame containing the input
        data.
    :param field: the name of the RAMSTK database field to retrieve the
        data for.
    :param default: the default value to assign to the field.
    :return: _value
    :rtype: the value of the requested input field or the default.
    """
    try:
        _value = df_row.at[mapper[field]]
    except KeyError:
        _value = default

    _value = _do_replace_nan(_value, default)

    # If it's supposed to be a date, make it a date.
    if default == date.today():
        _value = parser.parse(_value)

    return _value


class Import:
    """Contains the methods for importing data to a program database."""

    # Ordered dictionaries of RAMSTK inputs field headers.  Must use
    # OrderedDict to ensure they are always in the same order during program
    # flow.
    _dic_field_map = {
        "Function": OrderedDict(
            [
                ("Revision ID", ""),
                ("Function ID", ""),
                ("Level", ""),
                ("Function Code", ""),
                ("Function Name", ""),
                ("Parent", ""),
                ("Remarks", ""),
                ("Safety Critical", ""),
                ("Type", ""),
            ]
        ),
        "Requirement": OrderedDict(
            [
                ("Revision ID", ""),
                ("Requirement ID", ""),
                ("Derived?", ""),
                ("Requirement", ""),
                ("Figure Number", ""),
                ("Owner", ""),
                ("Page Number", ""),
                ("Parent ID", ""),
                ("Priority", ""),
                ("Requirement Code", ""),
                ("Specification", ""),
                ("Requirement Type", ""),
                ("Validated?", ""),
                ("Validated Date", ""),
            ]
        ),
        "Hardware": OrderedDict(
            [
                ("Revision ID", ""),
                ("Hardware ID", ""),
                ("Alternate Part Number", ""),
                ("CAGE Code", ""),
                ("Category ID", ""),
                ("Composite Ref. Des.", ""),
                ("Cost", ""),
                ("Cost Type", ""),
                ("Description", ""),
                ("Duty Cycle", ""),
                ("Figure Number", ""),
                ("LCN", ""),
                ("Level", ""),
                ("Manufacturer", ""),
                ("Mission Time", ""),
                ("Name", ""),
                ("NSN", ""),
                ("Page Number", ""),
                ("Parent Assembly", ""),
                ("Part", ""),
                ("Part Number", ""),
                ("Quantity", ""),
                ("Reference Designator", ""),
                ("Remarks", ""),
                ("Repairable", ""),
                ("Specification", ""),
                ("Subcategory ID", ""),
                ("Tagged Part", ""),
                ("Year of Manufacture", ""),
            ]
        ),
        "Design Electric": OrderedDict(
            [
                ("Hardware ID", ""),
                ("Application ID", ""),
                ("Area", ""),
                ("Capacitance", ""),
                ("Configuration ID", ""),
                ("Construction ID", ""),
                ("Contact Form ID", ""),
                ("Contact Gauge", ""),
                ("Contact Rating ID", ""),
                ("Current Operating", ""),
                ("Current Rated", ""),
                ("Current Ratio", ""),
                ("Environment Active ID", ""),
                ("Environment Dormant ID", ""),
                ("Family ID", ""),
                ("Feature Size", ""),
                ("Frequency Operating", ""),
                ("Insert ID", ""),
                ("Insulation ID", ""),
                ("Manufacturing ID", ""),
                ("Matching ID", ""),
                ("N Active Pins", ""),
                ("N Circuit Planes", ""),
                ("N Cycles", ""),
                ("N Elements", ""),
                ("N Hand Soldered", ""),
                ("N Wave Soldered", ""),
                ("Operating Life", ""),
                ("Overstress", ""),
                ("Package ID", ""),
                ("Power Operating", ""),
                ("Power Rated", ""),
                ("Power Ratio", ""),
                ("Reason", ""),
                ("Resistance", ""),
                ("Specification ID", ""),
                ("Technology ID", ""),
                ("Temperature, Active", ""),
                ("Temperature, Case", ""),
                ("Temperature, Dormant", ""),
                ("Temperature, Hot Spot", ""),
                ("Temperature, Junction", ""),
                ("Temperature, Knee", ""),
                ("Temperature, Rated Max", ""),
                ("Temperature, Rated Min", ""),
                ("Temperature Rise", ""),
                ("Theta JC", ""),
                ("Type ID", ""),
                ("Voltage, AC Operating", ""),
                ("Voltage, DC Operating", ""),
                ("Voltage ESD", ""),
                ("Voltage, Rated", ""),
                ("Voltage Ratio", ""),
                ("Weight", ""),
                ("Years in Production", ""),
            ]
        ),
        "Design Mechanic": OrderedDict(
            [
                ("Hardware ID", ""),
                ("Altitude, Operating", ""),
                ("Application ID", ""),
                ("Balance ID", ""),
                ("Clearance", ""),
                ("Casing ID", ""),
                ("Contact Pressure", ""),
                ("Deflection", ""),
                ("Diameter, Coil", ""),
                ("Diameter, Inner", ""),
                ("Diameter, Outer", ""),
                ("Diameter, Wire", ""),
                ("Filter Size", ""),
                ("Flow, Design", ""),
                ("Flow, Operating", ""),
                ("Frequency, Operating", ""),
                ("Friction", ""),
                ("Impact ID", ""),
                ("Allowable Leakage", ""),
                ("Length", ""),
                ("Length, Compressed", ""),
                ("Length, Relaxed", ""),
                ("Design Load", ""),
                ("Load ID", ""),
                ("Operating Load", ""),
                ("Lubrication ID", ""),
                ("Manufacturing ID", ""),
                ("Material ID", ""),
                ("Meyer Hardness", ""),
                ("Misalignment Angle", ""),
                ("N Ten", ""),
                ("N Cycles", ""),
                ("N Elements", ""),
                ("Offset", ""),
                ("Particle Size", ""),
                ("Contact Pressure", ""),
                ("Differential Pressure", ""),
                ("Downstream Pressure", ""),
                ("Rated Pressure", ""),
                ("Upstream Pressure", ""),
                ("Design RPM", ""),
                ("Operating RPM", ""),
                ("Service ID", ""),
                ("Spring Index", ""),
                ("Surface Finish", ""),
                ("Technology ID", ""),
                ("Thickness", ""),
                ("Torque ID", ""),
                ("Type ID", ""),
                ("Design Viscosity", ""),
                ("Dynamic Viscosity", ""),
                ("% Water", ""),
                ("Minimum Width", ""),
            ]
        ),
        "Reliability": OrderedDict(
            [
                ("Hardware ID", ""),
                ("Additive Adjustment Factor", ""),
                ("Failure Distribution ID", ""),
                ("Failure Rate Method ID", ""),
                ("Failure Rate Model", ""),
                ("Specified Failure Rate", ""),
                ("Failure Rate Type ID", ""),
                ("Location Parameter", ""),
                ("Specified MTBF", ""),
                ("Multiplicative Adjustment Factor", ""),
                ("Quality ID", ""),
                ("Reliability Goal", ""),
                ("Reliability Goal Measure ID", ""),
                ("Scale Parameter", ""),
                ("Shape Parameter", ""),
                ("Survival Analysis ID", ""),
            ]
        ),
        "Validation": OrderedDict(
            [
                ("Revision ID", ""),
                ("Validation ID", ""),
                ("Acceptable Maximum", ""),
                ("Acceptable Mean", ""),
                ("Acceptable Minimum", ""),
                ("Acceptable Variance", ""),
                ("s-Confidence", ""),
                ("Average Task Cost", ""),
                ("Maximum Task Cost", ""),
                ("Minimum Task Cost", ""),
                ("Start Date", ""),
                ("End Date", ""),
                ("Task Description", ""),
                ("Unit of Measure", ""),
                ("Name", ""),
                ("Task Status", ""),
                ("Task Type", ""),
                ("Task Specification", ""),
                ("Average Task Time", ""),
                ("Maximum Task Time", ""),
                ("Minimum Task Time", ""),
            ]
        ),
    }

    def __init__(self) -> None:
        """Initialize an ImportProject module instance."""
        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._dao: BaseDatabase = BaseDatabase()
        self._df_input_data: pd.DataFrame = pd.DataFrame({})

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_connect, "succeed_connect_program_database")
        pub.subscribe(self._do_map_to_field, "request_map_to_field")
        pub.subscribe(self._do_read_db_fields, "request_db_fields")
        pub.subscribe(self._do_read_file, "request_read_import_file")
        pub.subscribe(self._do_import, "request_import")

    def _do_connect(self, dao: BaseDatabase) -> None:
        """Connect data manager to a database.

        :param dao: the BaseDatabase() instance (data access object)
            representing the connected RAMSTK Program database.
        """
        self._dao = dao

    def _do_import(self, module: str) -> None:
        """Insert a new entity to the RAMSTK db with values from external file.

        :param module: the name of the RAMSTK module to import.
        :return: None
        :rtype: None
        """
        _entities = []

        # pylint: disable=unused-variable
        for __, _row in self._df_input_data.iterrows():
            if module == "Function":
                _entity = self._do_insert_function(_row)
                _entities.append(_entity)
            elif module == "Requirement":
                _entity = self._do_insert_requirement(_row)
                _entities.append(_entity)
            elif module == "Hardware":
                _entity = self._do_insert_hardware(_row)
                _entities.append(_entity)
                _entity = self._do_insert_allocation(_row)
                _entities.append(_entity)
                _entity = self._do_insert_similar_item(_row)
                _entities.append(_entity)
                _entity = self._do_insert_design_electric(_row)
                _entities.append(_entity)
                _entity = self._do_insert_mil_hdbk_f(_row)
                _entities.append(_entity)
                _entity = self._do_insert_design_mechanic(_row)
                _entities.append(_entity)
                _entity = self._do_insert_nswc(_row)
                _entities.append(_entity)
                _entity = self._do_insert_reliability(_row)
                _entities.append(_entity)
            elif module == "Validation":
                _entity = self._do_insert_validation(_row)
                _entities.append(_entity)

        try:
            self._dao.do_insert_many(_entities)  # type: ignore
            pub.sendMessage(
                "succeed_import_module",
                module=module,
            )
        except (AttributeError, DataAccessError):
            _method_name: str = inspect.currentframe().f_code.co_name  # type: ignore
            _error_msg: str = (
                "{1}: There was a problem importing {0} records.  "
                "This is usually caused by key violations; check the "
                "ID and/or parent ID fields in the import file."
            ).format(module, _method_name)
            pub.sendMessage(
                "fail_import_module",
                error_message=_error_msg,
            )

    def _do_insert_allocation(self, row: pd.Series) -> RAMSTKAllocationRecord:
        """Insert a new Allocation record to the RAMSTK database.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _allocation; an instance of the RAMSTKAllocationRecord database
            table record.
        :rtype: :class:`ramstk.models.programdb.RAMSTKAllocationRecord`
        """
        _allocation = RAMSTKAllocationRecord()
        _map = self._dic_field_map["Hardware"]

        _allocation.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _allocation.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)
        _allocation.parent_id = _get_input_value(_map, row, "Parent Assembly", 1)

        return _allocation

    def _do_insert_design_electric(self, row: pd.Series) -> RAMSTKDesignElectricRecord:
        """Insert a new Design Electric entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.RAMMSTKDesignElectric`
        """
        _design_electric = RAMSTKDesignElectricRecord()

        _map = self._dic_field_map["Hardware"]

        _design_electric.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)

        _map = self._dic_field_map["Design Electric"]
        _design_electric.set_attributes(
            {
                "application_id": _get_input_value(_map, row, "Application ID", 0),
                "area": _get_input_value(_map, row, "Area", 0.0),
                "capacitance": _get_input_value(_map, row, "Capacitance", 0.000001),
                "configuration_id": _get_input_value(_map, row, "Configuration ID", 0),
                "construction_id": _get_input_value(_map, row, "Construction ID", 0),
                "contact_form_id": _get_input_value(_map, row, "Contact Form ID", 0),
                "contact_gauge": _get_input_value(_map, row, "Contact Gauge", 20),
                "contact_rating_id": _get_input_value(
                    _map, row, "Contact Rating ID", 0
                ),
                "current_operating": _get_input_value(
                    _map, row, "Current Operating", 0.0
                ),
                "current_rated": _get_input_value(_map, row, "Current Rated", 0.0),
                "current_ratio": _get_input_value(_map, row, "Current Ratio", 0.0),
                "environment_active_id": _get_input_value(
                    _map, row, "Environment Active ID", 0
                ),
                "environment_dormant_id": _get_input_value(
                    _map, row, "Environment Dormant ID", 0
                ),
                "family_id": _get_input_value(_map, row, "Family ID", 0),
                "feature_size": _get_input_value(_map, row, "Feature Size", 1.0),
                "frequency_operating": _get_input_value(
                    _map, row, "Frequency Operating", 0.0
                ),
                "insert_id": _get_input_value(_map, row, "Insert ID", 0),
                "insulation_id": _get_input_value(_map, row, "Insulation ID", 0),
                "manufacturing_id": _get_input_value(_map, row, "Manufacturing ID", 0),
                "matching_id": _get_input_value(_map, row, "Matching ID", 0),
                "n_active_pins": _get_input_value(_map, row, "N Active Pins", 0),
                "n_circuit_planes": _get_input_value(_map, row, "N Circuit Planes", 1),
                "n_cycles": _get_input_value(_map, row, "N Cycles", 0),
                "n_elements": _get_input_value(_map, row, "N Elements", 0),
                "n_hand_soldered": _get_input_value(_map, row, "N Hand Soldered", 0),
                "n_wave_soldered": _get_input_value(_map, row, "N Wave Soldered", 0),
                "operating_life": _get_input_value(_map, row, "Operating Life", 0.0),
                "overstress": _get_input_value(_map, row, "Overstress", 0),
                "package_id": _get_input_value(_map, row, "Package ID", 0),
                "power_operating": _get_input_value(_map, row, "Power Operating", 0.0),
                "power_rated": _get_input_value(_map, row, "Power Rated", 0.0),
                "power_ratio": _get_input_value(_map, row, "Power Ratio", 0.0),
                "reason": _get_input_value(_map, row, "Reason", ""),
                "resistance": _get_input_value(_map, row, "Resistance", 0.0),
                "specification_id": _get_input_value(_map, row, "Specification ID", 0),
                "technology_id": _get_input_value(_map, row, "Technology ID", 0),
                "temperature_active": _get_input_value(
                    _map, row, "Temperature, Active", 30.0
                ),
                "temperature_case": _get_input_value(
                    _map, row, "Temperature, Case", 0.0
                ),
                "temperature_dormant": _get_input_value(
                    _map, row, "Temperature, Dormant", 25.0
                ),
                "temperature_hot_spot": _get_input_value(
                    _map, row, "Temperature, Hot Spot", 0.0
                ),
                "temperature_junction": _get_input_value(
                    _map, row, "Temperature, Junction", 0.0
                ),
                "temperature_knee": _get_input_value(
                    _map, row, "Temperature, Knee", 25.0
                ),
                "temperature_rated_max": _get_input_value(
                    _map, row, "Temperature, Rated Max", 0.0
                ),
                "temperature_rated_min": _get_input_value(
                    _map, row, "Temperature, Rated Min", 0.0
                ),
                "temperature_rise": _get_input_value(
                    _map, row, "Temperature Rise", 0.0
                ),
                "theta_jc": _get_input_value(_map, row, "Theta JC", 0.0),
                "type_id": _get_input_value(_map, row, "Type ID", 0),
                "voltage_ac_operating": _get_input_value(
                    _map, row, "Voltage, AC Operating", 0.0
                ),
                "voltage_dc_operating": _get_input_value(
                    _map, row, "Voltage, DC Operating", 0.0
                ),
                "voltage_esd": _get_input_value(_map, row, "Voltage ESD", 0.0),
                "voltage_rated": _get_input_value(_map, row, "Voltage, Rated", 0.0),
                "voltage_ratio": _get_input_value(_map, row, "Voltage Ratio", 0.0),
                "weight": _get_input_value(_map, row, "Weight", 1.0),
                "years_in_production": _get_input_value(
                    _map, row, "Years in Production", 2
                ),
            }
        )

        return _design_electric

    def _do_insert_design_mechanic(self, row: pd.Series) -> RAMSTKDesignMechanicRecord:
        """Insert a new Design Mechanic entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.RAMMSTKDesignMechanic`
        """
        _design_mechanic = RAMSTKDesignMechanicRecord()

        _map = self._dic_field_map["Hardware"]
        _design_mechanic.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)

        _map = self._dic_field_map["Design Mechanic"]
        _design_mechanic.set_attributes(
            {
                "altitude_operating": _get_input_value(
                    _map, row, "Altitude, Operating", 0.0
                ),
                "application_id": _get_input_value(_map, row, "Application ID", 0),
                "balance_id": _get_input_value(_map, row, "Balance ID", 0),
                "clearance": _get_input_value(_map, row, "Clearance", 0.0),
                "casing_id": _get_input_value(_map, row, "Casing ID", 0),
                "contact_pressure": _get_input_value(
                    _map, row, "Contact Pressure", 0.0
                ),
                "deflection": _get_input_value(_map, row, "Deflection", 0.0),
                "diameter_coil": _get_input_value(_map, row, "Diameter, Coil", 0.0),
                "diameter_inner": _get_input_value(_map, row, "Diameter, Inner", 0.0),
                "diameter_outer": _get_input_value(_map, row, "Diameter, Outer", 0.0),
                "diameter_wire": _get_input_value(_map, row, "Diameter, Wire", 0.0),
                "filter_size": _get_input_value(_map, row, "Filter Size", 0.0),
                "flow_design": _get_input_value(_map, row, "Flow, Design", 0.0),
                "flow_operating": _get_input_value(_map, row, "Flow, Operating", 0.0),
                "frequency_operating": _get_input_value(
                    _map, row, "Frequency, Operating", 0.0
                ),
                "friction": _get_input_value(_map, row, "Friction", 0.0),
                "impact_id": _get_input_value(_map, row, "Impact ID", 0),
                "leakage_allowable": _get_input_value(
                    _map, row, "Allowable Leakage", 0.0
                ),
                "length": _get_input_value(_map, row, "Length", 0.0),
                "length_compressed": _get_input_value(
                    _map, row, "Length, Compressed", 0.0
                ),
                "length_relaxed": _get_input_value(_map, row, "Length, Relaxed", 0.0),
                "load_design": _get_input_value(_map, row, "Design Load", 0.0),
                "load_id": _get_input_value(_map, row, "Load ID", 0),
                "load_operating": _get_input_value(_map, row, "Operating Load", 0.0),
                "lubrication_id": _get_input_value(_map, row, "Lubrication ID", 0),
                "manufacturing_id": _get_input_value(_map, row, "Manufacturing ID", 0),
                "material_id": _get_input_value(_map, row, "Material ID", 0),
                "meyer_hardness": _get_input_value(_map, row, "Meyer Hardness", 0.0),
                "misalignment_angle": _get_input_value(
                    _map, row, "Misalignment Angle", 0.0
                ),
                "n_ten": _get_input_value(_map, row, "N Ten", 0),
                "n_cycles": _get_input_value(_map, row, "N Cycles", 0.0),
                "n_elements": _get_input_value(_map, row, "N Elements", 0),
                "offset": _get_input_value(_map, row, "Offset", 0.0),
                "particle_size": _get_input_value(_map, row, "Particle Size", 0.0),
                "pressure_contact": _get_input_value(
                    _map, row, "Contact Pressure", 0.0
                ),
                "pressure_delta": _get_input_value(
                    _map, row, "Differential Pressure", 0.0
                ),
                "pressure_downstream": _get_input_value(
                    _map, row, "Downstream Pressure", 0.0
                ),
                "pressure_rated": _get_input_value(_map, row, "Rated Pressure", 0.0),
                "pressure_upstream": _get_input_value(
                    _map, row, "Upstream Pressure", 0.0
                ),
                "rpm_design": _get_input_value(_map, row, "Design RPM", 0.0),
                "rpm_operating": _get_input_value(_map, row, "Operating RPM", 0.0),
                "service_id": _get_input_value(_map, row, "Service ID", 0),
                "spring_index": _get_input_value(_map, row, "Spring Index", 0),
                "surface_finish": _get_input_value(_map, row, "Surface Finish", 0.0),
                "technology_id": _get_input_value(_map, row, "Technology ID", 0),
                "thickness": _get_input_value(_map, row, "Thickness", 0.0),
                "torque_id": _get_input_value(_map, row, "Torque ID", 0),
                "type_id": _get_input_value(_map, row, "Type ID", 0),
                "viscosity_design": _get_input_value(
                    _map, row, "Design Viscosity", 0.0
                ),
                "viscosity_dynamic": _get_input_value(
                    _map, row, "Dynamic Viscosity", 0.0
                ),
                "water_per_cent": _get_input_value(_map, row, "% Water", 0.0),
                "width_minimum": _get_input_value(_map, row, "Minimum Width", 0.0),
            }
        )

        return _design_mechanic

    def _do_insert_function(self, row: pd.Series) -> RAMSTKFunctionRecord:
        """Insert a new Function entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.ramskfunction.RAMSTKFunction`
        """
        _function = RAMSTKFunctionRecord()
        _map = self._dic_field_map["Function"]

        _function.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _function.function_id = _get_input_value(_map, row, "Function ID", 1)
        _function.function_code = _get_input_value(_map, row, "Function Code", "")
        _function.level = _get_input_value(_map, row, "Level", 0)
        _function.name = _get_input_value(_map, row, "Function Name", "")
        _function.parent_id = _get_input_value(_map, row, "Parent", 1)
        _function.remarks = _get_input_value(_map, row, "Remarks", "")
        _function.safety_critical = _get_input_value(_map, row, "Safety Critical", 0)
        _function.type_id = _get_input_value(_map, row, "Type", "")

        return _function

    def _do_insert_hardware(self, row: pd.Series) -> RAMSTKHardwareRecord:
        """Insert a new Hardware entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.ramstkhardware.RAMSTKHardware`
        """
        _hardware = RAMSTKHardwareRecord()
        _map = self._dic_field_map["Hardware"]

        _hardware.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _hardware.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)
        _hardware.alt_part_number = _get_input_value(
            _map, row, "Alternate Part Number", ""
        )
        _hardware.cage_code = _get_input_value(_map, row, "CAGE Code", "")
        _hardware.category_id = _get_input_value(_map, row, "Category ID", 0)
        _hardware.comp_ref_des = _get_input_value(_map, row, "Composite Ref. Des.", "")
        _hardware.cost = _get_input_value(_map, row, "Cost", 0.0)
        _hardware.cost_type_id = _get_input_value(_map, row, "Cost Type", 0)
        _hardware.description = _get_input_value(_map, row, "Description", "")
        _hardware.duty_cycle = _get_input_value(_map, row, "Duty Cycle", 100.0)
        _hardware.figure_number = _get_input_value(_map, row, "Figure Number", "")
        _hardware.lcn = _get_input_value(_map, row, "LCN", "")
        _hardware.level = _get_input_value(_map, row, "Level", 0)
        _hardware.manufacturer_id = _get_input_value(_map, row, "Manufacturer", 0)
        _hardware.mission_time = _get_input_value(_map, row, "Mission Time", 24.0)
        _hardware.name = _get_input_value(_map, row, "Name", "")
        _hardware.nsn = _get_input_value(_map, row, "NSN", "")
        _hardware.page_number = _get_input_value(_map, row, "Page Number", "")
        _hardware.parent_id = _get_input_value(_map, row, "Parent Assembly", 1)
        _hardware.part = _get_input_value(_map, row, "Part", 0)
        _hardware.part_number = _get_input_value(_map, row, "Part Number", "")
        _hardware.quantity = _get_input_value(_map, row, "Quantity", 1)
        _hardware.ref_des = _get_input_value(_map, row, "Reference Designator", "")
        _hardware.remarks = _get_input_value(_map, row, "Remarks", "")
        _hardware.repairable = _get_input_value(_map, row, "Repairable", 1)
        _hardware.specification_number = _get_input_value(
            _map, row, "Specification", ""
        )
        _hardware.subcategory_id = _get_input_value(_map, row, "Subcategory ID", 0)
        _hardware.tagged_part = _get_input_value(_map, row, "Tagged Part", 0)
        _hardware.year_of_manufacture = _get_input_value(
            _map, row, "Year of Manufacture", 1900
        )

        return _hardware

    def _do_insert_mil_hdbk_f(self, row: pd.Series) -> RAMSTKMilHdbk217FRecord:
        """Insert a new MIL-HDBK-217F entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.RAMSTKMilHdbkF`
        """
        _mil_hdbk_f = RAMSTKMilHdbk217FRecord()

        _map = self._dic_field_map["Hardware"]
        _mil_hdbk_f.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)

        return _mil_hdbk_f

    def _do_insert_nswc(self, row: pd.Series) -> RAMSTKNSWCRecord:
        """Insert a new NSWC entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.nswc.RAMSTKNSWCRecord`
        """
        _nswc = RAMSTKNSWCRecord()

        _map = self._dic_field_map["Hardware"]
        _nswc.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)

        return _nswc

    def _do_insert_reliability(self, row: pd.Series) -> RAMSTKReliabilityRecord:
        """Insert a new Reliability entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.RAMSTKReliability`
        """
        _reliability = RAMSTKReliabilityRecord()

        _map = self._dic_field_map["Hardware"]
        _reliability.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)

        _map = self._dic_field_map["Reliability"]
        _reliability.add_adj_factor = _get_input_value(
            _map, row, "Additive Adjustment Factor", 0.0
        )
        _reliability.failure_distribution_id = _get_input_value(
            _map, row, "Failure Distribution ID", 0
        )
        _reliability.hazard_rate_method_id = _get_input_value(
            _map, row, "Failure Rate Method ID", 0
        )
        _reliability.hazard_rate_model = _get_input_value(
            _map, row, "Failure Rate Model", ""
        )
        _reliability.hazard_rate_specified = _get_input_value(
            _map, row, "Specified Failure Rate", 0.0
        )
        _reliability.hazard_rate_type_id = _get_input_value(
            _map, row, "Failure Rate Type ID", 0
        )
        _reliability.location_parameter = _get_input_value(
            _map, row, "Location Parameter", 0.0
        )
        _reliability.mtbf_specified = _get_input_value(_map, row, "Specified MTBF", 0.0)
        _reliability.mult_adj_factor = _get_input_value(
            _map, row, "Multiplicative Adjustment Factor", 1.0
        )
        _reliability.quality_id = _get_input_value(_map, row, "Quality ID", 0)
        _reliability.reliability_goal = _get_input_value(
            _map, row, "Reliability Goal", 100.0
        )
        _reliability.reliability_goal_measure_id = _get_input_value(
            _map, row, "Reliability Goal Measure ID", 0
        )
        _reliability.scale_parameter = _get_input_value(
            _map, row, "Scale Parameter", 0.0
        )
        _reliability.shape_parameter = _get_input_value(
            _map, row, "Shape Parameter", 0.0
        )
        _reliability.survival_analysis_id = _get_input_value(
            _map, row, "Survival Analysis ID", 0
        )

        return _reliability

    def _do_insert_requirement(self, row: pd.Series) -> RAMSTKRequirementRecord:
        """Insert a new Requirement entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input data.
        :return: _entity
        :rtype: :class:`ramstk.models.programdb.RAMSTKRequirement`
        """
        _requirement = RAMSTKRequirementRecord()
        _map = self._dic_field_map["Requirement"]

        _requirement.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _requirement.requirement_id = _get_input_value(_map, row, "Requirement ID", 1)
        _requirement.derived = _get_input_value(_map, row, "Derived?", 0)
        _requirement.description = _get_input_value(_map, row, "Requirement", "")
        _requirement.figure_number = _get_input_value(_map, row, "Figure Number", "")
        _requirement.owner = _get_input_value(_map, row, "Owner", "")
        _requirement.page_number = _get_input_value(_map, row, "Page Number", "")
        _requirement.parent_id = _get_input_value(_map, row, "Parent ID", 1)
        _requirement.priority = _get_input_value(_map, row, "Priority", 1)
        _requirement.requirement_code = _get_input_value(
            _map, row, "Requirement Code", ""
        )
        _requirement.specification = _get_input_value(_map, row, "Specification", "")
        _requirement.requirement_type = _get_input_value(
            _map, row, "Requirement Type", ""
        )
        _requirement.validated = _get_input_value(_map, row, "Validated?", 0)
        _requirement.validated_date = _get_input_value(
            _map, row, "Validated Date", date.today()
        )

        return _requirement

    def _do_insert_similar_item(self, row: pd.Series) -> RAMSTKSimilarItemRecord:
        """Insert a new Similar Item record to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input
            data.
        :return: _similar_item; an instance of the RAMSTKSimilarItem database
            table record.
        :rtype: :class:`ramstk.models.programdb.RAMSTKSimilarItem`
        """
        _similar_item = RAMSTKSimilarItemRecord()
        _map = self._dic_field_map["Hardware"]

        _similar_item.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _similar_item.hardware_id = _get_input_value(_map, row, "Hardware ID", 1)
        _similar_item.parent_id = _get_input_value(_map, row, "Parent Assembly", 1)

        return _similar_item

    def _do_insert_validation(self, row: pd.Series) -> RAMSTKValidationRecord:
        """Insert a new Validation entity to the RAMSTK db.

        :param row: the row from the pandas DataFrame containing the input data.
        :return: _entity
        :rtype: :class:`ramstk.dao.programdb.RAMSTKValidation.RAMSTKValidation`
        """
        _validation = RAMSTKValidationRecord()
        _map = self._dic_field_map["Validation"]

        _validation.revision_id = _get_input_value(_map, row, "Revision ID", 1)
        _validation.validation_id = _get_input_value(_map, row, "Validation ID", 1)
        _validation.acceptable_maximum = _get_input_value(
            _map, row, "Acceptable Maximum", 0.0
        )
        _validation.acceptable_mean = _get_input_value(
            _map, row, "Acceptable Mean", 0.0
        )
        _validation.acceptable_minimum = _get_input_value(
            _map, row, "Acceptable Minimum", 0.0
        )
        _validation.acceptable_variance = _get_input_value(
            _map, row, "Acceptable Variance", 0.0
        )
        _validation.confidence = _get_input_value(_map, row, "s-Confidence", 75.0)
        _validation.cost_average = _get_input_value(_map, row, "Average Task Cost", 0.0)
        _validation.cost_maximum = _get_input_value(_map, row, "Maximum Task Cost", 0.0)
        _validation.cost_minimum = _get_input_value(_map, row, "Minimum Task Cost", 0.0)
        _validation.date_start = _get_input_value(_map, row, "Start Date", date.today())
        _validation.date_end = _get_input_value(_map, row, "End Date", date.today())
        _validation.description = _get_input_value(_map, row, "Task Description", "")
        _validation.measurement_unit = _get_input_value(
            _map, row, "Unit of Measure", ""
        )
        _validation.name = _get_input_value(_map, row, "Name", "")
        _validation.status = _get_input_value(_map, row, "Task Status", 0.0)
        _validation.task_type = _get_input_value(_map, row, "Task Type", "")
        _validation.task_specification = _get_input_value(
            _map, row, "Task Specification", ""
        )
        _validation.time_average = _get_input_value(_map, row, "Average Task Time", 0.0)
        _validation.time_maximum = _get_input_value(_map, row, "Maximum Task Time", 0.0)
        _validation.time_minimum = _get_input_value(_map, row, "Minimum Task Time", 0.0)

        return _validation

    def _do_map_to_field(
        self, module: str, import_field: str, format_field: str
    ) -> None:
        """Map the external column to the RAMSTK database table field.

        :param module: the RAMSTK module to map header fields for.
        :param import_field: the string used for the column header in the
            import file.
        :param format_field: the string used for default titles in the
            RAMSTK layout file.
        :return: None
        :rtype: None
        """
        self._dic_field_map[module][format_field] = import_field

    def _do_read_db_fields(self, module: str) -> None:
        """Return the database field names in a list.

        :param module: the name of the work stream module to return
            database field names for.
        :return: None
        :rtype: None
        """
        _db_fields = []
        for _field in self._dic_field_map[module]:
            _db_fields.append(_field)

        pub.sendMessage(
            "succeed_read_db_fields",
            db_fields=_db_fields,
        )

    def _do_read_file(self, file_type: str, file_name: str) -> None:
        """Read contents of input file into a pandas DataFrame().

        :param file_type: the type of file to import from.  Supported files
            types are:
                - CSV (using a semi-colon (;) delimiter)
                - Excel
        :param file_name: the name, with full path, of the file to export
            the RAMSTK Program database data to.
        :return: None
        :rtype: None
        """
        if file_type == "csv":
            self._df_input_data = pd.read_csv(
                file_name, sep=";", na_values=[""], parse_dates=True
            )
        elif file_type == "text":
            self._df_input_data = pd.read_csv(
                file_name, sep=" ", na_values=[""], parse_dates=True
            )
        elif file_type == "excel":
            self._df_input_data = pd.read_excel(file_name)

        pub.sendMessage(
            "succeed_read_import_file",
            import_fields=list(self._df_input_data.axes[1].tolist()[1:]),
        )



# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK common database models package."""

# RAMSTK Local Imports
from .ramstkcategory import RAMSTKCategory
from .ramstkcondition import RAMSTKCondition
from .ramstkfailuremode import RAMSTKFailureMode
from .ramstkgroup import RAMSTKGroup
from .ramstkhazards import RAMSTKHazards
from .ramstkloadhistory import RAMSTKLoadHistory
from .ramstkmanufacturer import RAMSTKManufacturer
from .ramstkmeasurement import RAMSTKMeasurement
from .ramstkmethod import RAMSTKMethod
from .ramstkmodel import RAMSTKModel
from .ramstkrpn import RAMSTKRPN
from .ramstkstakeholders import RAMSTKStakeholders
from .ramstkstatus import RAMSTKStatus
from .ramstksubcategory import RAMSTKSubCategory
from .ramstktype import RAMSTKType
from .ramstkuser import RAMSTKUser
